import pytest
from u2s_sdk.client import ApiClient
from u2s_sdk.handler import ResumableUploadHandler

@pytest.fixture
def client(mocker):
    return ApiClient(base_url='https://test-api.com')

def test_start_upload_successful(client, mocker):
    # Mock the post method of the ApiClient to simulate a successful start_upload request
    mocker.patch.object(client, 'post', return_value=mocker.Mock(status_code=201, headers={'Location': '/upload?key=123'}))

    handler = ResumableUploadHandler(client)
    location_uri, upload_key = handler.start_upload('test.txt', 1024)

    assert location_uri == '/upload?key=123'
    assert upload_key == '123'

def test_start_upload_failed(client, mocker):
    # Mock the post method of the ApiClient to simulate a failed start_upload request
    mocker.patch.object(client, 'post', return_value=mocker.Mock(status_code=400))

    handler = ResumableUploadHandler(client)

    with pytest.raises(Exception, match=r'Failed to initiate resumable upload\. Status code: 400'):
        handler.start_upload('test.txt', 1024)

def test_upload_chunk_successful(client, mocker):
    # Mock the put method of the ApiClient to simulate a successful upload_chunk request
    mocker.patch.object(client, 'put', return_value=mocker.Mock(status_code=201, headers={}))

    handler = ResumableUploadHandler(client)
    response_headers = handler.upload_chunk('/upload?key=123', 4, b'data', 0, 3)

    assert response_headers == {}

def test_upload_chunk_in_progress(client, mocker):
    # Mock the put method of the ApiClient to simulate an in-progress upload_chunk request
    mocker.patch.object(client, 'put', return_value=mocker.Mock(status_code=308))

    handler = ResumableUploadHandler(client)
    response_headers = handler.upload_chunk('/upload?key=123', 4, b'data', 0, 2)

    assert response_headers is False

def test_upload_chunk_failed(client, mocker):
    # Mock the put method of the ApiClient to simulate a failed upload_chunk request
    mocker.patch.object(client, 'put', return_value=mocker.Mock(status_code=400))

    handler = ResumableUploadHandler(client)

    with pytest.raises(Exception, match=r'Failed to upload chunk\. Status code: 400'):
        handler.upload_chunk('/upload?key=123', 1024, b'data', 0, 3)

# Add more test cases for the ResumableUploadHandler class as needed



"""
u2s_sdk.utils
"""
from urllib.parse import urlparse, parse_qs


def get_key_value_from_uri(uri: str):
    """
    Get the key value from a URI.
    :param uri: The URI
    :return: The upload key value
    """
    # Parse the URI
    parsed_uri = urlparse(uri)

    # Get the query parameters as a dictionary
    query_parameters = parse_qs(parsed_uri.query)

    # Retrieve the value of the "key" parameter
    key_value = query_parameters.get("key", [None])[0]

    return key_value



"""
Client module.

u2s_sdk.client
"""
import logging
import requests


class ApiClient:
    """
    API client.
    """
    def __init__(self, base_url='https://api.up2sha.re', timeout=10, api_key=None, oauth_token=None):
        """
        Constructor.

        :param base_url: The base URL (default: https://api.up2sha.re)
        :param timeout: The timeout (in seconds)
        :param api_key: The API key (optional)
        :param oauth_token: The OAuth token (optional)
        """
        self.base_url = base_url
        self.timeout = timeout
        self.api_key = api_key
        self.oauth_token = oauth_token

        # default logging level and format
        self.set_logging()

    # a method that defines the request logging level and format
    def set_logging(self, level: int =logging.DEBUG, fmt: str ='%(asctime)s - %(name)s - %(levelname)s - %(message)s'):
        """
        Set logging.

        :param level: The logging level (default: logging.DEBUG)
        :param fmt: The logging format (default: '%(asctime)s - %(name)s - %(levelname)s - %(message)s')
        """
        logger = logging.getLogger(__name__)
        logger.setLevel(level)
        # Create a console handler
        console_handler = logging.StreamHandler()
        # Set the formatter for the console handler (customize this as needed)
        formatter = logging.Formatter(fmt)
        console_handler.setFormatter(formatter)
        # Add the console handler to the logger
        logger.addHandler(console_handler)

        # Store the logger in the class
        self.logger = logger
        self._set_requests_logging(level, fmt)

    # method to get the logger
    def get_logger(self):
        """
        Get the logger.

        :return: The logger
        """
        return self.logger

    # same for the request library
    def _set_requests_logging(self, level: int, fmt: str):
        """
        Set logging for the requests library.

        :meta private:

        :param level: The logging level
        :param fmt: The logging format
        """
        logging.getLogger("requests").setLevel(level)
        logging.getLogger("urllib3").setLevel(level)
        # Create a console handler
        console_handler = logging.StreamHandler()
        # Set the formatter for the console handler (customize this as needed)
        formatter = logging.Formatter(fmt)
        console_handler.setFormatter(formatter)
        # Add the console handler to the logger
        logging.getLogger("requests").addHandler(console_handler)
        logging.getLogger("urllib3").addHandler(console_handler)

        # from http.client import HTTPConnection  # py3

        # # print statements from `http.client.HTTPConnection` to console/stdout
        # HTTPConnection.debuglevel = 1

    def get_headers(self):
        """
        Get the headers.

        :return: The headers
        """
        headers = {
            # 'Content-Type': 'application/json',
            'Accept': 'application/json'
        }

        if self.api_key:
            headers['X-Api-Key'] = self.api_key

        if self.oauth_token:
            headers['Authorization'] = f'Bearer {self.oauth_token}'

        return headers

    def get(self, endpoint: str, headers=None):
        """
        Perform a GET request.

        :param endpoint: The endpoint
        :param headers: The headers
        :return: The response
        """
        return self._make_request('GET', endpoint, headers=headers)

    def post(self, endpoint:str , data=None, headers=None):
        """
        Perform a POST request.

        :param endpoint: The endpoint
        :param data: The data
        :param headers: The headers
        :return: The response
        """
        return self._make_request('POST', endpoint, data=data, headers=headers)

    def put(self, endpoint: str, data=None, headers=None):
        """
        Perform a PUT request.

        :param endpoint: The endpoint
        :param data: The data
        :param headers: The headers
        :return: The response
        """
        return self._make_request('PUT', endpoint, data=data, headers=headers)

    def delete(self, endpoint: str, headers=None):
        """
        Perform a DELETE request.

        :param endpoint: The endpoint
        :param headers: The headers
        :return: The response
        """
        return self._make_request('DELETE', endpoint, headers=headers)

    def _make_request(self, method: str, endpoint: str, data=None, headers=None):
        """
        Make a request.

        :param method: The method
        :param endpoint: The endpoint
        :param data: The data
        :param headers: The headers
        :return: The response
        """
        url = f'{self.base_url}/{endpoint}'
        _headers = self.get_headers()

        # Merge the headers
        if headers:
            _headers.update(headers)
        headers = _headers

        response = None

        try:
            if method == 'GET':
                response = requests.get(url, headers=headers, timeout=self.timeout)
            elif method == 'POST':
                response = requests.post(url, headers=headers, data=data, timeout=self.timeout)
            elif method == 'PUT':
                response = requests.put(url, headers=headers, data=data, timeout=self.timeout)
            elif method == 'DELETE':
                response = requests.delete(url, headers=headers, timeout=self.timeout)

            response.raise_for_status()

        except requests.exceptions.RequestException as e:
            # print(f'Error: {e}')
            self.logger.error(f'Error: {e}')
            return None

        # return response.json()
        return response



from u2s_sdk.utils import get_key_value_from_uri

def test_get_key_value_from_uri_with_valid_key():
    uri = "https://example.com/path?key=value"
    result = get_key_value_from_uri(uri)
    assert result == "value"

def test_get_key_value_from_uri_with_missing_key():
    uri = "https://example.com/path?other=value"
    result = get_key_value_from_uri(uri)
    assert result is None

# def test_get_key_value_from_uri_with_multiple_keys():
#     uri = "https://example.com/path?key=value1&key=value2"
#     result = get_key_value_from_uri(uri)
#     assert result == "value1"

def test_get_key_value_from_uri_with_empty_uri():
    uri = ""
    result = get_key_value_from_uri(uri)
    assert result is None



import os

from u2s_sdk.handler import ResumableUploadHandler
from u2s_sdk.client import ApiClient


def main():
    api_client = ApiClient('https://api.up2sha.re', api_key='your_api_key', timeout=30)

    # Initialize the ResumableUploadHandler
    handler = ResumableUploadHandler(api_client)

    # Example usage for resumable upload with specified chunk size
    file_name = 'file_example_MP4_1920_18MG.mp4'
    file_path = os.path.join(os.path.dirname(__file__), file_name)
    chunk_size = 1048576  # 1 MB

    with open(file_path, 'rb') as file:
        handler.simulate_chunk_upload(file, chunk_size, filename=file_name)


if __name__ == "__main__":
    main()



import pytest
from u2s_sdk.client import ApiClient
import requests.exceptions

@pytest.fixture
def client(mocker):
    return ApiClient(base_url='https://test-api.com')

def test_get_headers_no_auth(client):
    headers = client.get_headers()
    assert 'X-Api-Key' not in headers
    assert 'Authorization' not in headers

def test_get_headers_api_key():
    client = ApiClient(base_url='https://test-api.com', api_key='test_key')
    headers = client.get_headers()
    assert headers['X-Api-Key'] == 'test_key'
    assert 'Authorization' not in headers

def test_get_headers_oauth_token():
    client = ApiClient(base_url='https://test-api.com', oauth_token='test_token')
    headers = client.get_headers()
    assert 'X-Api-Key' not in headers
    assert headers['Authorization'] == 'Bearer test_token'

def test_get_request_successful(client, mocker):
    # Mock the requests.get method
    mock_get = mocker.patch('requests.get')
    mock_response = mocker.MagicMock()
    mock_response.status_code = 200
    mock_get.return_value = mock_response

    response = client.get('/test')
    assert response.status_code == 200

def test_get_request_exception(client, mocker):
    # Mock the requests.get method to raise an exception
    mocker.patch('requests.get', side_effect=requests.exceptions.RequestException('Test error'))

    response = client.get('/test')
    assert response is None



"""
File module.

u2s_sdk.file
"""
from .client import ApiClient


class FileHandler:
    """
    File handler.
    """
    def __init__(self, api_client: ApiClient):
        """
        Constructor.

        :param api_client: The API client
        """
        self.api_client = api_client
        self.logger = api_client.get_logger()
        self.limit = 100
        self.include = 'owner'
        self.search_join = 'and'

    def create_download_token(self, file_id: int):
        """
        Create a download token for a file.

        :param file_id: The file ID
        :return: The download token
        """
        endpoint = f'/files/{str(file_id)}/downloadtoken'
        response = self.api_client.get(endpoint)
        if response.status_code == 200:
            return response.json()
        else:
            # Handle error response here
            return None

    def get_raw(self, file_id: int, range_header: str, token: str, dl_token: str, dl_expiry: int|str):
        """
        Get the raw file data.

        :param file_id: The file ID
        :param range_header: The range header
        :param token: The token
        :param dl_token: The download token
        :param dl_expiry: The download expiry
        :return: The raw file data
        """
        endpoint = f'/files/{str(file_id)}/raw'
        headers = {
            'Range': range_header,
        }
        params = {
            'token': token,
            'dl-token': dl_token,
            'dl-expiry': dl_expiry,
        }
        response = self.api_client.get(endpoint, headers=headers, params=params)
        if response.status_code == 200:
            return response.content
        elif response.status_code == 206:
            # Handle partial content response here
            return response.content
        else:
            # Handle error response here
            return None

    def list(self, include=None, search=None, limit=None, search_join=None):
        """
        List files.

        :param include: The include parameter (owner|uploadKey|shares|activity)
        :param search: The search parameter
        :param limit: The limit parameter
        :param search_join: The search join parameter (or|and)
        :return: The file list
        """

        # Validate include parameter
        if include is not None:
            if include not in ['owner', 'uploadKey', 'shares', 'activity']:
                raise Exception('Invalid include parameter')
        else:
            include = self.include

        # Validate search_join parameter
        if search_join is not None:
            if search_join not in ['or', 'and']:
                raise Exception('Invalid search_join parameter')
        else:
            search_join = self.search_join

        # Validate limit parameter
        if limit is not None:
            if limit < 1 or limit > 100:
                raise Exception('Invalid limit parameter')
        else:
            limit = self.limit

        endpoint = '/files'
        params = {
            'include': include,
            'search': search,
            'limit': limit,
            'searchJoin': search_join,
        }
        response = self.api_client.get(endpoint, params=params)
        if response.status_code == 200:
            return response.json()
        else:
            # Handle error response here
            return None

    def delete(self, file_id: int):
        """
        Delete a file.

        :param file_id: The file ID
        :return: True if the deletion was successful, False otherwise
        """
        endpoint = f'/files/{str(file_id)}'
        response = self.api_client.delete(endpoint)
        if response.status_code == 204:
            # Deletion successful
            return True
        else:
            # Handle error response here
            return False

    def update(self, file_id: int, filename=None, description=None, visibility=None):
        """
        Update a file.

        :param file_id: The file ID
        :param filename: The filename
        :param description: The description
        :param visibility: The visibility
        :return: The updated file
        """
        endpoint = f'/files/{str(file_id)}'
        data = {
            'filename': filename,
            'description': description,
            'visibility': visibility,
        }
        response = self.api_client.put(endpoint, json=data)
        if response.status_code == 200:
            return response.json()
        else:
            # Handle error response here
            return None



from setuptools import setup, find_packages

with open('requirements.txt') as f:
    requirements = f.read().splitlines()

setup(
    name='u2s_sdk',
    version='0.3.0',
    description='Up2Share SDK for Python',
    author='Corentin Chepeau',
    author_email='contact+python@up2sha.re',
    packages=find_packages(),
    install_requires=requirements,
)



"""
Handler module.

u2s_sdk.handler
"""
import os
import json

from .utils import get_key_value_from_uri
from .client import ApiClient


class ResumableUploadHandler:
    """
    Resumable upload handler.
    """

    def __init__(self, api_client: ApiClient):
        """
        Constructor.

        :param api_client: The API client
        """
        self.api_client = api_client
        self.logger = api_client.get_logger()

    def start_upload(self, filename: str, total_size: int, content_type='application/octet-stream'):
        """
        Start a resumable upload.

        :param filename: The filename
        :param total_size: The total file size (in bytes)
        :param content_type: The content type (default: application/octet-stream)
        :return: The upload URI and the upload key
        """
        headers = {
            'Content-Type': content_type,
            'X-Upload-Content-Length': str(total_size),
            'X-Upload-Content-Type': 'application/octet-stream'
        }

        # Step 1: Initiate the resumable upload
        endpoint = '/files#resumable'
        data = {
            'filename': filename,
        }
        response = self.api_client.post(endpoint, data=json.dumps(data), headers=headers)

        if response.status_code == 201:
            location_uri = response.headers['Location']
            self.logger.debug(f'Location URI: {location_uri}')
            upload_key = get_key_value_from_uri(location_uri)
            self.logger.info('Resumable upload initiated successfully')
            return location_uri, upload_key
        else:
            raise Exception(f'Failed to initiate resumable upload. Status code: {response.status_code}')

    def upload_chunk(self, upload_uri: str, total_size: int, chunk_data: bytes, chunk_start: int, chunk_end: int):
        """
        Upload a chunk of data.

        :param upload_uri: The upload URI (obtained from the response of the start_upload method)
        :param total_size: The total file size (in bytes)
        :param chunk_data: The chunk data (bytes)
        :param chunk_start: The start position of the chunk (0-based)
        :param chunk_end: The end position of the chunk (0-based)
        :return: Response headers (includes location) if the upload was successful, False otherwise
        """
        self.logger.info(f'Uploading chunk {chunk_start}-{chunk_end}/{total_size}')

        headers = {
            'Content-Range': f'bytes {chunk_start}-{chunk_end}/{total_size}',
            'Content-Type': 'application/octet-stream',
            'Content-Length': str(chunk_end - chunk_start + 1),
        }

        # Step 2: Upload the chunk
        response = self.api_client.put(upload_uri, data=chunk_data, headers=headers)

        self.logger.info(f'Response status code: {response.status_code}')

        self.logger.debug(f'Response headers: {response.headers}')
        self.logger.debug(f'Response text: {response.text}')

        if response.status_code == 201:
            # return True  # Chunk uploaded successfully
            return response.headers
        elif response.status_code == 308:
            return False  # Chunk upload is still in progress
        else:
            self.logger.error(response.text)
            raise Exception(f'Failed to upload chunk. Status code: {response.status_code}')

    def simulate_chunk_upload(self, buffer: bytes, chunk_size=5242880, filename=None):
        """
        Simulate chunk upload.

        :param buffer: The file buffer
        :param chunk_size: The chunk size in bytes (default: 5 MB)
        :param filename: The filename (default: random filename)
        :return: Response headers (includes location) if the upload was successful, False otherwise
        """
        try:
            # Seek to the end of the file
            buffer.seek(0, 2)  # 0 indicates the offset and 2 indicates seeking from the end of the file
            # Get the current file position, which is the length of the file
            total_size = buffer.tell()
            buffer.seek(0)  # Reset the file position to the beginning of the file

            self.logger.info(f'Total file size: {total_size}')

            if not filename:
                # Use a random filename
                filename = os.urandom(16).hex()

            upload_uri, upload_key = self.start_upload(filename, total_size)

            chunk_start = 0
            while chunk_start < (total_size - 1):
                chunk_end = min(chunk_start + chunk_size - 1, total_size - 1)
                chunk_data = buffer.read(chunk_end - chunk_start + 1)
                success = self.upload_chunk(upload_uri, total_size, chunk_data, chunk_start, chunk_end)

                chunk_start = chunk_end + 1

                if success:
                    # We are done here
                    self.logger.info('Response headers: ', success)
                    # break
                    pass

            self.logger.info('Resumable upload completed successfully')
            return success
        except Exception as e:
            self.logger.error(f'Error: {e}')
            # raise e
            return False
